@@ -11,6 +11,7 @@ from tests.environ import EnvironmentContext
 import os
 
 from zim.fs import File, Dir
+from zim.newfs.mock import os_native_path
 from zim.config import *
 from zim.notebook import Path
 
@@ -45,8 +46,8 @@ zim.config.data_dir = marshal_path_lookup(zim.config.data_dir)
 
 class FilterInvalidConfigWarning(tests.LoggingFilter):
 
-	logger = 'zim.config'
-	message = 'Invalid config value'
+	def __init__(self):
+		tests.LoggingFilter.__init__(self, 'zim.config', 'Invalid config value')
 
 
 class EnvironmentConfigContext(EnvironmentContext):
@@ -514,61 +515,6 @@ none=
 		self.assertFalse(conf.modified)
 
 
-class TestHeaders(tests.TestCase):
-
-	def runTest(self):
-		'''Test HeadersDict class'''
-		# normal operation
-		text='''\
-Foobar: 123
-More-Lines: test
-	1234
-	test
-Aaa: foobar
-'''
-		headers = HeadersDict(text)
-		self.assertEqual(headers['Foobar'], '123')
-		self.assertEqual(headers['More-Lines'], 'test\n1234\ntest')
-		self.assertEqual(headers.dump(), text.splitlines(True))
-
-		moretext='''\
-Foobar: 123
-More-Lines: test
-	1234
-	test
-Aaa: foobar
-
-test 123
-test 456
-'''
-		lines = moretext.splitlines(True)
-		headers = HeadersDict()
-		headers.read(lines)
-		self.assertEqual(headers.dump(), text.splitlines(True))
-		self.assertEqual(lines, ['test 123\n', 'test 456\n'])
-
-		# error tolerance and case insensitivity
-		text = '''\
-more-lines: test
-1234
-test
-'''
-		self.assertRaises(HeaderParsingError, HeadersDict, text)
-
-		text = '''\
-fooo
-more-lines: test
-1234
-test
-'''
-		self.assertRaises(HeaderParsingError, HeadersDict, text)
-
-		text = 'foo-bar: test\n\n\n'
-		headers = HeadersDict(text)
-		self.assertEqual(headers['Foo-Bar'], 'test')
-		self.assertEqual(headers.dump(), ['Foo-Bar: test\n'])
-
-
 class TestUserDirs(tests.TestCase):
 
 	def setUp(self):
@@ -761,6 +707,8 @@ foo=myprofile
 	}
 
 	def assertMatchPath(self, file, path):
+		if not file.path.startswith('<virtual>'):
+			path = os_native_path(path)
 		assert file.path.endswith(path), '"%s" does not match "%s"' % (file.path, path)
 
 	def runTest(self):
@@ -850,5 +798,3 @@ class TestConfigManager(tests.TestCase, ConfigManagerTests):
 
 		self.manager = ConfigManager()
 		self.prefix = 'zim'
-
-
