@@ -11,7 +11,7 @@ import tests
 
 from zim.formats import *
 from zim.fs import File
-from zim.notebook import Path, Link
+from zim.notebook import Path
 from zim.parsing import link_type
 from zim.templates import Template
 
@@ -19,9 +19,6 @@ from zim.templates import Template
 if not ElementTreeModule.__name__.endswith('cElementTree'):
 	print 'WARNING: using ElementTree instead of cElementTree'
 
-wikitext = tests.WikiTestData.get('roundtrip')
-
-
 
 class TestFormatMixin(object):
 	'''Mixin for testing formats, uses data in C{tests/data/formats/}'''
@@ -75,14 +72,16 @@ class TestFormatMixin(object):
 		# Dumper
 		wanted = self.getReferenceData()
 		reftree = tests.new_parsetree_from_xml(self.reference_xml)
-		linker = StubLinker()
-		linker.set_base(Dir('tests/data/formats'))
+		linker = StubLinker(Dir('tests/data/formats'))
 		dumper = self.format.Dumper(linker=linker)
 		result = ''.join(dumper.dump(reftree))
 		#~ print '\n' + '>'*80 + '\n' + result + '\n' + '<'*80 + '\n'
 		self.assertMultiLineEqual(result, wanted)
 		self.assertNoTextMissing(result, reftree)
 
+		# Check that dumper did not modify the tree
+		self.assertMultiLineEqual(reftree.tostring(), self.reference_xml)
+
 		# partial dumper
 		parttree = tests.new_parsetree_from_xml("<?xml version='1.0' encoding='utf-8'?>\n<zim-tree partial=\"True\">try these <strong>bold</strong>, <emphasis>italic</emphasis></zim-tree>")
 		result = ''.join(dumper.dump(parttree))
@@ -115,7 +114,7 @@ class TestFormatMixin(object):
 		# TODO how to handle objects ??
 		assert isinstance(text, basestring)
 		offset = 0
-		for elt in tree.iter():
+		for elt in tree._etree.iter():
 			if elt.tag == 'img':
 				elttext = (elt.tail) # img text is optional
 			else:
@@ -179,8 +178,6 @@ class TestParseTree(tests.TestCase):
 		tree = ParseTree()
 		r = tree.fromstring(self.xml)
 		self.assertEqual(id(r), id(tree)) # check return value
-		e = tree.getroot()
-		self.assertEqual(e.tag, 'zim-tree') # check content
 		text = tree.tostring()
 		self.assertEqual(text, self.xml)
 
@@ -256,7 +253,6 @@ class TestParseTree(tests.TestCase):
 		text = tree.tostring()
 		self.assertEqual(text, wanted)
 
-
 	def testGetEndsWithNewline(self):
 		for xml, newline in (
 			('<zim-tree partial="True">foo</zim-tree>', False),
@@ -271,6 +267,65 @@ class TestParseTree(tests.TestCase):
 			tree = ParseTree().fromstring(xml)
 			self.assertEqual(tree.get_ends_with_newline(), newline)
 
+	def testGetObjects(self):
+		xml = File('tests/data/formats/parsetree.xml').read().rstrip('\n')
+		tree = tests.new_parsetree_from_xml(xml)
+		objects = list(tree.get_objects())
+		self.assertTrue(len(objects) >= 2)
+
+	def testFindall(self):
+		tree = ParseTree().fromstring(self.xml)
+		wanted = [
+			(1, 'Head 1'),
+			(2, 'Head 2'),
+			(3, 'Head 3'),
+			(2, 'Head 4'),
+			(5, 'Head 5'),
+			(4, 'Head 6'),
+			(5, 'Head 7'),
+			(6, 'Head 8'),
+		]
+		found = []
+		for elt in tree.findall(HEADING):
+			found.append((int(elt.get('level')), elt.gettext()))
+		self.assertEqual(found, wanted)
+
+	def testReplace(self):
+		def replace(elt):
+			# level 2 becomes 3
+			# level 3 is replaced by text
+			# level 4 is removed
+			# level 5 is skipped
+			# level 1 and 6 stay as is
+			level = int(elt.get('level'))
+			if level == 2:
+				elt.attrib['level'] = 3
+				return elt
+			elif level == 3:
+				return DocumentFragment(*elt)
+			elif level == 4:
+				return None
+			elif level == 5:
+				raise VisitorSkip
+			else:
+				return elt
+		tree = ParseTree().fromstring(self.xml)
+		wanted = '''\
+<?xml version='1.0' encoding='utf-8'?>
+<zim-tree>
+<h level="1">Head 1</h>
+<h level="3">Head 2</h>
+Head 3
+<h level="3">Head 4</h>
+<h level="5">Head 5</h>
+
+<h level="5">Head 7</h>
+<h level="6">Head 8</h>
+</zim-tree>'''
+		tree.replace(HEADING, replace)
+		text = tree.tostring()
+		self.assertEqual(text, wanted)
+
 
 class TestTextFormat(tests.TestCase, TestFormatMixin):
 
@@ -315,21 +370,22 @@ A list
 	* baz
 '''
 		tree = self.format.Parser().parse(input)
+		#~ print tree.tostring()
 		output = self.format.Dumper().dump(tree)
-		self.assertEqual(output, text.splitlines(True))
+		self.assertEqual(''.join(output), text)
 
 	def testLink(self):
 		'''Test iterator function for link'''
 		# + check for bugs in link encoding
-		text = '[[FooBar]] [[Foo|]] [[|Foo]]'
+		text = '[[FooBar]] [[Foo|]] [[|Foo]] [[||]]'
 		tree = self.format.Parser().parse(text)
-		for i, tag in enumerate(tree.getiterator('link')):
-			self.assertTrue(tag.text)
-			self.assertTrue(tag.attrib.get('href'))
-			link = Link(self.page, **tag.attrib)
-			self.assertEqual(tag.attrib['href'], link.href)
-			done = True
-		self.assertEqual(i, 2)
+		#~ print tree.tostring()
+		found = 0
+		for elt in tree.findall(LINK):
+			self.assertTrue(elt.gettext())
+			self.assertTrue(elt.get('href'))
+			found += 1
+		self.assertEqual(found, 3)
 
 	def testBackward(self):
 		'''Test backward compatibility for wiki format'''
@@ -550,16 +606,89 @@ class TestHtmlFormat(tests.TestCase, TestFormatMixin):
 
 	def testEncoding(self):
 		'''Test HTML encoding'''
-		page = Element('zim-tree')
-		para = SubElement(page, 'p')
-		para.text = '<foo>"foo" & "bar"</foo>'
-		tree = ParseTree(page)
+		builder = ParseTreeBuilder()
+		builder.start(FORMATTEDTEXT)
+		builder.append(PARAGRAPH, None, '<foo>"foo" & "bar"</foo>\n')
+		builder.end(FORMATTEDTEXT)
+		tree = builder.get_parsetree()
 		html = self.format.Dumper(linker=StubLinker()).dump(tree)
-		self.assertEqual(html,
-			['<p>\n', '&lt;foo&gt;"foo" &amp; "bar"&lt;/foo&gt;</p>\n'] )
+		self.assertEqual(''.join(html),
+			'<p>\n&lt;foo&gt;"foo" &amp; "bar"&lt;/foo&gt;\n</p>\n')
 
 	# TODO add test using http://validator.w3.org
 
+	def testEmptyLines(self):
+		builder = ParseTreeBuilder()
+		builder.start(FORMATTEDTEXT)
+		builder.append(HEADING, {'level':1}, 'head1')
+		builder.text('\n')
+		builder.append(HEADING, {'level':2}, 'head2')
+		builder.end(FORMATTEDTEXT)
+		tree = builder.get_parsetree()
+
+		html = self.format.Dumper(
+			linker=StubLinker(),
+			template_options={'empty_lines': 'default'}
+		).dump(tree)
+		self.assertEqual(''.join(html),
+			'<h1>head1</h1>\n\n'
+			'<br>\n\n'
+			'<h2>head2</h2>\n\n'
+		)
+
+		html = self.format.Dumper(
+			linker=StubLinker(),
+			template_options={'empty_lines': 'remove'}
+		).dump(tree)
+		self.assertEqual(''.join(html),
+			'<h1>head1</h1>\n\n'
+			'<h2>head2</h2>\n\n'
+		)
+
+		html = self.format.Dumper(
+			linker=StubLinker(),
+			template_options={'empty_lines': 'Remove'} # case sensitive
+		).dump(tree)
+		self.assertEqual(''.join(html),
+			'<h1>head1</h1>\n\n'
+			'<h2>head2</h2>\n\n'
+		)
+
+
+	def testLineBreaks(self):
+		builder = ParseTreeBuilder()
+		builder.start(FORMATTEDTEXT)
+		builder.append(PARAGRAPH, None,
+			'bla bla bla\n'
+			'bla bla bla\n'
+		)
+		builder.end(FORMATTEDTEXT)
+		tree = builder.get_parsetree()
+
+		html = self.format.Dumper(
+			linker=StubLinker(),
+			template_options={'line_breaks': 'default'}
+		).dump(tree)
+		self.assertEqual(''.join(html),
+			'<p>\n'
+			'bla bla bla<br>\n'
+			'bla bla bla\n'
+			'</p>\n'
+		)
+
+		html = self.format.Dumper(
+			linker=StubLinker(),
+			template_options={'line_breaks': 'remove'}
+		).dump(tree)
+		self.assertEqual(''.join(html),
+			'<p>\n'
+			'bla bla bla\n'
+			'bla bla bla\n'
+			'</p>\n'
+		)
+
+
+
 class TestMarkdownFormat(tests.TestCase, TestFormatMixin):
 
 	def setUp(self):
@@ -574,8 +703,12 @@ class TestRstFormat(tests.TestCase, TestFormatMixin):
 
 class LatexLoggingFilter(tests.LoggingFilter):
 
-	logger = 'zim.formats.latex'
-	message = ('No document type set in template', 'Could not find latex equation')
+	def __init__(self):
+		tests.LoggingFilter.__init__(
+			self,
+			'zim.formats.latex',
+			('No document type set in template', 'Could not find latex equation')
+		)
 
 
 class TestLatexFormat(tests.TestCase, TestFormatMixin):
@@ -602,66 +735,43 @@ class TestLatexFormat(tests.TestCase, TestFormatMixin):
 
 		input = r'\foo $ % ^ \% bar < >'
 		wanted = r'$\backslash$foo \$  \% \^{} $\backslash$\% bar \textless{} \textgreater{}'
-		self.assertEqual(format.tex_encode(input), wanted)
+		self.assertEqual(format.Dumper.encode_text(PARAGRAPH, input), wanted)
 
-	def testExport(self):
-		'''test the export of a wiki page to latex'''
-		with LatexLoggingFilter():
-			format = get_format('LaTeX')
-			testpage = tests.WikiTestData.get('Test:wiki')
-			tree = get_format('wiki').Parser().parse(testpage)
-			output = format.Dumper(linker=StubLinker()).dump(tree)
-			#~ print '>>>\n' + ''.join(output) + '<<<'
-			self.assertTrue('\chapter{Foo Bar}\n' in output)
+	def testDocumentType(self):
+		builder = ParseTreeBuilder()
+		builder.start(FORMATTEDTEXT)
+		builder.append(HEADING, {'level':1}, 'head1')
+		builder.text('\n')
+		builder.append(HEADING, {'level':2}, 'head2')
+		builder.end(FORMATTEDTEXT)
+		tree = builder.get_parsetree()
 
-		# Test template_options.document_type
-		input = r'''
-[% options.document_type = 'book' -%]
-\title{[% page.basename %]}
-
-\begin{document}
-\maketitle
-\tableofcontents
-[% page.body %]
-\end{document}
-'''
-		wanted = r'''
-\title{FooBar}
-
-\begin{document}
-\maketitle
-\tableofcontents
-\textbf{foo bar !}
+		for type, head1 in (
+			('report', 'chapter'),
+			('article', 'section'),
+			('book', 'part'),
+		):
+			lines = self.format.Dumper(
+				linker=StubLinker(),
+				template_options={'document_type': type}
+			).dump(tree)
+			self.assertIn(head1, ''.join(lines))
 
 
+class StubFile(object):
 
-\chapter{Heading 2}
+	def __init__(self, path, text):
+		self.path = path
+		self.text = text
 
-duss
+	def read(self):
+		return self.text
 
 
-\end{document}
-'''
-
-		notebook = tests.new_notebook()
-		page = notebook.get_page(Path('FooBar'))
-		page.parse('wiki', '''\
-====== Page Heading ======
-**foo bar !**
-
-===== Heading 2 =====
-duss
-''')
-
-		template = Template(input, 'latex', linker=StubLinker())
-		result = template.process(notebook, page)
-		self.assertEqual(''.join(result), wanted)
-
-
-class TestParseTreeBuilder(tests.TestCase):
+class TestOldParseTreeBuilder(tests.TestCase):
 
 	def runTest(self):
-		'''Test ParseTreeBuilder class'''
+		'''Test OldParseTreeBuilder class'''
 		# - Test \n before and after h / p / pre
 		# - Test break line into lines
 		input = '''\
@@ -713,8 +823,33 @@ grrr
 
 		# For some reason this does not work with cElementTree.XMLBuilder ...
 		from xml.etree.ElementTree import XMLTreeBuilder
-		builder = XMLTreeBuilder(target=ParseTreeBuilder())
+		builder = XMLTreeBuilder(target=OldParseTreeBuilder())
 		builder.feed(input)
 		root = builder.close()
 		tree = ParseTree(root)
 		self.assertEqual(tree.tostring(), wanted)
+
+
+class TestParseHeaderLines(tests.TestCase):
+
+	def runTest(self):
+		text = '''\
+Content-Type: text/x-zim-wiki
+Wiki-Format: zim 0.4
+X-Foo: Some text
+	here
+Creation-Date: 2010-12-14T14:15:09.134955
+
+Blaat
+'''
+		body, meta = parse_header_lines(text)
+		self.assertEqual(dict(meta), {
+			'Content-Type': 'text/x-zim-wiki',
+			'Wiki-Format': 'zim 0.4',
+			'Creation-Date': '2010-12-14T14:15:09.134955',
+			'X-Foo': 'Some text\nhere'
+		})
+		self.assertEqual(body, 'Blaat\n')
+
+		out = dump_header_lines(meta)
+		self.assertEqual(out+'\nBlaat\n', text)
