@@ -6,6 +6,7 @@ from __future__ import with_statement
 
 import tests
 
+import os
 
 from zim.fs import File, Dir
 from zim.formats import wiki, ParseTree
@@ -14,11 +15,13 @@ from zim.gui.pageview import *
 from zim.config import SectionedConfigDict, VirtualConfigManager, ConfigManager
 from zim.gui.clipboard import Clipboard
 
+from zim.newfs.mock import os_native_path
+
 
 class FilterNoSuchImageWarning(tests.LoggingFilter):
 
-	logger = 'zim.gui.pageview'
-	message = 'No such image:'
+	def __init__(self):
+		tests.LoggingFilter.__init__(self, 'zim.gui.pageview', 'No such image:')
 
 
 def new_parsetree_from_text(text):
@@ -34,13 +37,6 @@ def new_parsetree_from_text(text):
 
 def setUpPageView(fakedir=None, notebook=None):
 	'''Some bootstrap code to get an isolated PageView object'''
-	## TODO - should not be needed
-	## we can get rid of this when we refactor the actiongroup stuff
-	## to not register, by be called by the window
-	PageView.actiongroup = tests.MockObject() # use class attribute to fake ui init
-	PageView.actiongroup.mock_method('get_action', tests.MockObject())
-	PageView.actiongroup.mock_method('list_actions', [])
-
 	if notebook is None:
 		notebook = tests.new_notebook(fakedir)
 
@@ -48,13 +44,60 @@ def setUpPageView(fakedir=None, notebook=None):
 	ui.config = VirtualConfigManager()
 	ui.notebook = notebook
 	ui.page = None
-	ui.uimanager = tests.MockObject()
-	ui.uimanager.mock_method('get_accel_group', tests.MockObject())
+	#~ ui.uimanager = tests.MockObject()
+	#~ ui.uimanager.mock_method('get_accel_group', tests.MockObject())
 
-	ui.mainwindow = tests.MockObject()
-	ui.mainwindow.statusbar_style_label = tests.MockObject()
+	return PageView(ui, ui.notebook)
+
+
+class TestLines(tests.TestCase):
+
+	def testLines(self):
+		'''Test lines formatting.'''
+
+		pageview = setUpPageView()
+		buffer = pageview.view.get_buffer()
+
+		def check_text(input, result):
+			buffer.set_text(input)
+			tree = buffer.get_parsetree()
+			dumper = get_format('wiki').Dumper()
+			text = ''.join(dumper.dump(tree))
+			self.assertEqual(text, result)
+
+		# Check formatting.
+		string = 'text... \n{}\n text... \n'
+		input = string.format('-'*4)
+		check_text(input, input) # doesn't format
+		for i in range(30):
+			if i < 5:
+				output = string.format('-'*i)
+			else:
+				output = string.format(LINE_TEXT)
+			input = string.format('-'*i)
+			check_text(input, output)
+
+		# Check that any additional symbol other than '-' fails.
+		input = 'text... {}\n text... \n'.format('-'*10)
+		check_text(input, input)
+		input = 'text... \n{}text... \n'.format('-'*10)
+		check_text(input, input)
+		input = 'text... \n{} \n text... \n'.format('-'*10)
+		check_text(input, input)
+		input = 'text... \n {}\n text... \n'.format('-'*10)
+		check_text(input, input)
+
+		# Check more complex text.
+		string = 'text... \n\n{0}\n\n{0}\n\n text... \n'
+		input = string.format('-'*7)
+		output = string.format(LINE_TEXT)
+		check_text(input, output)
+
+		string = '... \n{}\n{}\n{}\n ... \n{}\n{}0\n'
+		input = string.format('-'*8, '-'*6, '-'*4, '-'*11, '-'*10)
+		output = string.format(LINE_TEXT, LINE_TEXT, '-'*4, LINE_TEXT, '-'*10)
+		check_text(input, output)
 
-	return PageView(ui)
 
 
 class TestCaseMixin(object):
@@ -1610,11 +1653,12 @@ foo
 		page = tests.new_page_from_text('[[~//bar.txt]]')
 			# Extra '/' is in there to verify path gets parsed as File object
 		pageview.set_page(page)
+		wanted = '~/bar.txt' if os.name != 'nt' else '~\\bar.txt'
 		click(_('Copy _Link'))
 		self.assertEqual(Clipboard.get_text(), '~/bar.txt')
 		tree = Clipboard.get_parsetree(pageview.ui.notebook, page)
 		self.assertEqual(tree.tostring(),
-			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n<zim-tree><link href="~/bar.txt">~/bar.txt</link></zim-tree>')
+			'<?xml version=\'1.0\' encoding=\'utf-8\'?>\n<zim-tree><link href="%s">%s</link></zim-tree>' % (wanted, wanted))
 
 
 
@@ -1693,14 +1737,14 @@ Baz
 
 		buffer.place_cursor(buffer.get_end_iter())
 		pageview.insert_links((Path("foo"), File("/foo.txt"), "~/bar.txt"))
-		wantedtext = 'Test 123\nfoo\n%s\n~/bar.txt\n' % File('/foo.txt').uri
+		wantedtext = 'Test 123\nfoo\n%s\n%s\n' % (File('/foo.txt').uri, os_native_path('~/bar.txt'))
 		text = buffer.get_text(*buffer.get_bounds())
 		self.assertEqual(text, wantedtext)
 
 		buffer.place_cursor(buffer.get_iter_at_line(2))
 		buffer.select_line()
 		pageview.insert_links(('http://cpan.org',))
-		wantedtext = 'Test 123\nfoo\n%s\n~/bar.txt\n' % 'http://cpan.org '
+		wantedtext = 'Test 123\nfoo\n%s\n%s\n' % ('http://cpan.org ', os_native_path('~/bar.txt'))
 		text = buffer.get_text(*buffer.get_bounds())
 		self.assertEqual(text, wantedtext)
 
@@ -1729,9 +1773,10 @@ class TestPageviewDialogs(tests.TestCase):
 
 		## Insert Image dialog
 		ui = MockUI()
+		ui.notebook.dir = Dir(self.get_tmp_name())
 		buffer = MockBuffer()
 		file = File('data/zim.png')
-		dialog = InsertImageDialog(ui, buffer, Path(':some_page'), file)
+		dialog = InsertImageDialog(ui, buffer, ui.notebook, Path(':some_page'), file)
 		self.assertTrue(dialog.filechooser.get_preview_widget_active())
 		#~ self.assertEqual(dialog.get_file(), file)
 		#~ dialog.assert_response_ok()
@@ -1768,8 +1813,9 @@ class TestPageviewDialogs(tests.TestCase):
 
 		## Insert text from file dialog
 		ui = MockUI()
+		ui.notebook.dir = Dir(self.get_tmp_name())
 		buffer = MockBuffer()
-		dialog = InsertTextFromFileDialog(ui, buffer)
+		dialog = InsertTextFromFileDialog(ui, buffer, ui.notebook, Path(':some_page'))
 		#~ dialog.set_file()
 		#~ dialog.assert_response_ok()
 		#~ self.assertEqual(buffer.mock_calls[-1][0], 'insert_parsetree_at_cursor')
@@ -1803,9 +1849,9 @@ dus bar bazzz baz
 	def testInsertLinkDialog(self):
 		# Insert Link dialog
 		ui = MockUI()
-		ui.notebook.index = tests.MockObject()
-		ui.notebook.index.mock_method('list_pages', [])
-		ui.notebook.index.mock_method('walk', [])
+		ui.notebook.pages = tests.MockObject()
+		ui.notebook.pages.mock_method('list_pages', [])
+		ui.notebook.pages.mock_method('walk', [])
 		pageview = tests.MockObject()
 		pageview.page = Path('Test:foo:bar')
 		textview = TextView({})
@@ -1822,7 +1868,6 @@ class MockUI(tests.MockObject):
 
 	def __init__(self):
 		tests.MockObject.__init__(self)
-		self.mainwindow = None
 		self.notebook = tests.MockObject()
 		self.preferences = SectionedConfigDict()
 		self.page = Path('Test')
@@ -1886,3 +1931,69 @@ class TestCamelCase(tests.TestCase):
 				camelcase(unicode(text)),
 				msg='"%s" should NOT be CamelCase' % text
 			)
+
+
+@tests.skipIf(gtk.pygtk_version < (2, 10), 'old pygtk, no serialization formats')
+class TestDragAndDropFunctions(tests.TestCase):
+
+	def testSerializeParseTree(self):
+		tree = tests.new_parsetree()
+		tree.resolve_images()
+		buffer = TextBuffer()
+		with FilterNoSuchImageWarning():
+			buffer.insert_parsetree_at_cursor(tree)
+
+		start, end = buffer.get_bounds()
+		xml = buffer.serialize(buffer, 'text/x-zim-parsetree', start, end)
+		tree.unresolve_images()
+		tree._etree.getroot().attrib['partial'] = True # HACK
+		self.assertEqual(xml, tree.tostring())
+
+	def testDeserializeParseTree(self):
+		notebook = tests.MockObject()
+		path = Path('Mock')
+		buffer = TextBuffer(notebook, path)
+		iter = buffer.get_insert_iter()
+		xml = tests.new_parsetree().tostring()
+		with FilterNoSuchImageWarning():
+			buffer.deserialize(buffer, 'text/x-zim-parsetree', iter, xml)
+
+	def testDeserializeUriList(self):
+		notebook = self.setUpNotebook()
+		path = Path('Mock')
+		buffer = TextBuffer(notebook, path)
+
+		# external uris
+		iter = buffer.get_insert_iter()
+		data = "http://wikipedia.com\r\n"
+		buffer.deserialize(buffer, 'text/uri-list', iter, data)
+
+		tree = buffer.get_parsetree()
+		xml = tree.tostring()
+		self.assertIn('http://wikipedia.com', xml) # FIXME: should use tree api
+
+		# internal uris
+		iter = buffer.get_insert_iter()
+		data = "Foo:Bar\r\n"
+		buffer.deserialize(buffer, 'text/x-zim-page-list-internal', iter, data)
+
+		tree = buffer.get_parsetree()
+		xml = tree.tostring()
+		self.assertIn('Foo:Bar', xml) # FIXME: should use tree api
+
+	def testDeserializeImageData(self):
+		#folder = self.setUpFolder('imagedata', mock=tests.MOCK_NEVER)
+		folder = Dir(self.create_tmp_dir('imagedata'))
+		notebook = tests.MockObject()
+		notebook.mock_method('get_attachments_dir', folder)
+		notebook.resolve_file = lambda fpath, ppath: fpath
+		path = Path('Mock')
+
+		buffer = TextBuffer(notebook, path)
+		image = File('./data/zim.png').raw()
+		iter = buffer.get_insert_iter()
+		buffer.deserialize(buffer, 'image/png', iter, image)
+
+		tree = buffer.get_parsetree()
+		xml = tree.tostring()
+		self.assertIn("pasted_image.png", xml) # FIXME: should use tree api to get image
