@@ -1,21 +1,27 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2010 Jaap Karssenberg <jaap.karssenberg@gmail.com>
+# Copyright 2010-2014 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 import gtk
 
 import re
 from datetime import date as dateclass
 
-from zim.plugins import PluginClass
-from zim.config import get_config, data_file
-from zim.notebook import resolve_notebook, get_notebook, Notebook, PageNameError
-from zim.ipc import start_server_if_not_running, ServerProxy
+from zim.fs import Dir
+
+from zim.plugins import PluginClass, WindowExtension, extends
+from zim.main import GtkCommand
+from zim.actions import action
+from zim.config import data_file, ConfigManager
+from zim.notebook import Path, Notebook, NotebookInfo, \
+	resolve_notebook, build_notebook
 from zim.gui.widgets import Dialog, ScrolledTextView, IconButton, \
-	InputForm, gtk_window_set_default_icon, QuestionDialog
+	InputForm, QuestionDialog
 from zim.gui.clipboard import Clipboard, SelectionClipboard
 from zim.gui.notebookdialog import NotebookComboBox
-from zim.templates import GenericTemplate, StrftimeFunction
+from zim.templates import get_template
+
+from zim.main import ZIM_APPLICATION
 
 
 import logging
@@ -27,109 +33,117 @@ usagehelp = '''\
 usage: zim --plugin quicknote [OPTIONS]
 
 Options:
-  notebook=URI         Select the notebook in the dialog
-  page=STRING          Fill in full page name
-  namespace=STRING     Fill in the namespace in the dialog
-  basename=STRING      Fill in the page name in the dialog
-  append=[true|false]  Set whether to append or create new page
-  text=TEXT            Provide the text directly
-  input=stdin          Provide the text on stdin
-  input=clipboard      Take the text from the clipboard
-  encoding=base64      Text is encoded in base64
-  encoding=url         Text is url encoded
-                       (In both cases expects UTF-8 after decoding)
-  attachments=FOLDER   Import all files in FOLDER as attachments,
-                       wiki input can refer these files relatively
-  option:url=STRING    Set template parameter
+  --help, -h             Print this help text and exit
+  --notebook URI         Select the notebook in the dialog
+  --page STRING          Fill in full page name
+  --section STRING       Fill in the page section in the dialog
+  --basename STRING      Fill in the page name in the dialog
+  --append [true|false]  Set whether to append or create new page
+  --text TEXT            Provide the text directly
+  --input stdin          Provide the text on stdin
+  --input clipboard      Take the text from the clipboard
+  --encoding base64      Text is encoded in base64
+  --encoding url         Text is url encoded
+                         (In both cases expects UTF-8 after decoding)
+  --attachments FOLDER   Import all files in FOLDER as attachments,
+                         wiki input can refer these files relatively
+  --option url=STRING    Set template parameter
 '''
 
 
-def main(*args):
-	options = {}
-	template_options = {}
-	for arg in args:
-		if arg.startswith('option:'):
-			arg = arg[7:]
-			dict = template_options
-		else:
-			dict = options
+class QuickNotePluginCommand(GtkCommand):
 
-		if '=' in arg:
-			key, value = arg.split('=', 1)
-			dict[key] = value
-		else:
-			dict[arg] = True
-	#~ print 'OPTIONS:', options, template_options
-
-	if 'help' in options:
-		print usagehelp
-		return
-
-	if 'notebook' in options:
-		notebook, page = resolve_notebook(options['notebook'])
-	else:
-		notebook = None
-
-	if 'append' in options:
-		if options['append'].lower() == 'true':
-			options['append'] = True
-		else:
-			options['append'] = False
-
-	if 'input' in options:
-		if options['input'] == 'stdin':
-			import sys
-			text = sys.stdin.read()
-		elif options['input'] == 'clipboard':
-			text = \
-				SelectionClipboard.get_text() \
-				or Clipboard.get_text()
-	else:
-		text = options.get('text')
-
-	if text and options.get('encoding'):
-		if options['encoding'] == 'base64':
-			import base64
-			text = base64.b64decode(text)
-		elif options['encoding'] == 'url':
-			from zim.parsing import url_decode, URL_ENCODE_DATA
-			text = url_decode(text, mode=URL_ENCODE_DATA)
-		else:
-			raise AssertionError, 'Unknown encoding: %s' % options['encoding']
-
-	if text and not isinstance(text, unicode):
-		text = text.decode('utf-8')
-
-	icon = data_file('zim.png').path
-	gtk_window_set_default_icon()
-
-	dialog = QuickNoteDialog(None,
-		notebook,
-		options.get('namespace'), options.get('basename'),
-		options.get('append'),
-		text,
-		template_options,
-		options.get('attachments')
+	options = (
+		('help', 'h', 'Print this help text and exit'),
+		('notebook=', '', 'Select the notebook in the dialog'),
+		('page=', '', 'Fill in full page name'),
+		('section=', '', 'Fill in the page section in the dialog'),
+		('namespace=', '', 'Fill in the page section in the dialog'), # backward compatibility
+		('basename=', '', 'Fill in the page name in the dialog'),
+		('append=', '', 'Set whether to append or create new page ("true" or "false")'),
+		('text=', '', 'Provide the text directly'),
+		('input=', '', 'Provide the text on stdin ("stdin") or take the text from the clipboard ("clipboard")'),
+		('encoding=', '', 'Text encoding ("base64" or "url")'),
+		('attachments=', '', 'Import all files in FOLDER as attachments, wiki input can refer these files relatively'),
+		('option=', '', 'Set template parameter, e.g. "url=URL"'),
 	)
-	dialog.run()
 
+	def parse_options(self, *args):
+		self.opts['option'] = [] # allow list
 
-ui_actions = (
-	# name, stock id, label, accelerator, tooltip, read only
-	('show_quick_note', 'gtk-new', _('Quick Note...'), '', '', False), # T: menu item
-)
+		if all(not a.startswith('-') for a in args):
+			# Backward compartibility for options not prefixed by "--"
+			# used "=" as separator for values
+			# template options came as "option:KEY=VALUE"
+			for arg in args:
+				if arg.startswith('option:'):
+					self.opts['option'].append(arg[7:])
+				elif arg == 'help':
+					self.opts['help'] = True
+				else:
+					key, value = arg.split('=', 1)
+					self.opts[key] = value
+		else:
+			GtkCommand.parse_options(self, *args)
 
-ui_xml = '''
-<ui>
-	<menubar name='menubar'>
-		<menu action='file_menu'>
-			<placeholder name="open_items">
-				<menuitem action="show_quick_note" />
-			</placeholder>
-		</menu>
-	</menubar>
-</ui>
-'''
+		self.template_options = {}
+		for arg in self.opts['option']:
+			key, value = arg.split('=', 1)
+			self.template_options[key] = value
+
+		if 'append' in self.opts:
+			self.opts['append'] = \
+				self.opts['append'].lower() == 'true'
+
+	def get_text(self):
+		if 'input' in self.opts:
+			if self.opts['input'] == 'stdin':
+				import sys
+				text = sys.stdin.read()
+			elif self.opts['input'] == 'clipboard':
+				text = \
+					SelectionClipboard.get_text() \
+					or Clipboard.get_text()
+			else:
+				raise AssertionError, 'Unknown input type: %s' % self.opts['input']
+		else:
+			text = self.opts.get('text')
+
+		if text and 'encoding' in self.opts:
+			if self.opts['encoding'] == 'base64':
+				import base64
+				text = base64.b64decode(text)
+			elif self.opts['encoding'] == 'url':
+				from zim.parsing import url_decode, URL_ENCODE_DATA
+				text = url_decode(text, mode=URL_ENCODE_DATA)
+			else:
+				raise AssertionError, 'Unknown encoding: %s' % self.opts['encoding']
+
+		if text and not isinstance(text, unicode):
+			text = text.decode('utf-8')
+
+		return text
+
+	def run(self):
+		if self.opts.get('help'):
+			print usagehelp # TODO handle this in the base class
+		else:
+			if 'notebook' in self.opts:
+				notebook = resolve_notebook(self.opts['notebook'])
+			else:
+				notebook = None
+
+			dialog = QuickNoteDialog(None,
+				notebook=notebook,
+				namespace=self.opts.get('namespace'),
+				basename=self.opts.get('basename'),
+				append=self.opts.get('append'),
+				text=self.get_text(),
+				template_options=self.template_options,
+				attachments=self.opts.get('attachments')
+			)
+			dialog.show_all()
+			return dialog
 
 
 class QuickNotePlugin(PluginClass):
@@ -150,21 +164,39 @@ This is a core plugin shipping with zim.
 		# key, type, label, default
 	#~ )
 
-	def initialize_ui(self, ui):
-		if ui.ui_type == 'gtk':
-			ui.add_actions(ui_actions, self)
-			ui.add_ui(ui_xml, self)
 
+@extends('MainWindow')
+class MainWindowExtension(WindowExtension):
+
+	uimanager_xml = '''
+	<ui>
+		<menubar name='menubar'>
+			<menu action='file_menu'>
+				<placeholder name="open_items">
+					<menuitem action="show_quick_note" />
+				</placeholder>
+			</menu>
+		</menubar>
+	</ui>
+	'''
+
+	@action(_('Quick Note...'), stock='gtk-new') # T: menu item
 	def show_quick_note(self):
-		dialog = BoundQuickNoteDialog.unique(self, self.ui, {})
+		ui = self.window.ui # XXX
+		notebook = self.window.ui.notebook # XXX
+		dialog = BoundQuickNoteDialog.unique(self, self.window, notebook, ui)
 		dialog.show()
 
 
 class BoundQuickNoteDialog(Dialog):
 	'''Dialog bound to a specific notebook'''
 
-	def __init__(self, ui, page=None, namespace=None, basename=None, append=None, text=None, template_options=None, attachments=None):
-		Dialog.__init__(self, ui, _('Quick Note'))
+	def __init__(self, window, notebook, ui,
+		page=None, namespace=None, basename=None,
+		append=None, text=None, template_options=None, attachments=None
+	):
+		Dialog.__init__(self, window, _('Quick Note'))
+		self._ui = ui
 		self._updating_title = False
 		self._title_set_manually = not basename is None
 		self.attachments = attachments
@@ -172,7 +204,7 @@ class BoundQuickNoteDialog(Dialog):
 		self.uistate.setdefault('namespace', None, basestring)
 		namespace = namespace or self.uistate['namespace']
 
-		self.form = InputForm(notebook=self.ui.notebook)
+		self.form = InputForm(notebook=notebook)
 		self.vbox.pack_start(self.form, False)
 		self._init_inputs(namespace, basename, append, text, template_options)
 
@@ -189,7 +221,7 @@ class BoundQuickNoteDialog(Dialog):
 
 		self.form.add_inputs( (
 				('page', 'page', _('Page')),
-				('namespace', 'namespace', _('Namespace')), # T: text entry field
+				('namespace', 'namespace', _('Page section')), # T: text entry field
 				('new_page', 'bool', _('Create a new page for each note')), # T: checkbox in Quick Note dialog
 				('basename', 'string', _('Title')) # T: text entry field
 			) )
@@ -228,11 +260,11 @@ class BoundQuickNoteDialog(Dialog):
 		switch_input()
 		self.form.widgets['new_page'].connect('toggled', switch_input)
 
-		self.open_page = gtk.CheckButton(_('Open _Page')) # T: Option in quicknote dialog
+		self.open_page_check = gtk.CheckButton(_('Open _Page')) # T: Option in quicknote dialog
 			# Don't use "O" as accelerator here to avoid conflict with "Ok"
-		self.open_page.set_active(self.uistate['open_page'])
-		self.action_area.pack_start(self.open_page, False)
-		self.action_area.set_child_secondary(self.open_page, True)
+		self.open_page_check.set_active(self.uistate['open_page'])
+		self.action_area.pack_start(self.open_page_check, False)
+		self.action_area.set_child_secondary(self.open_page_check, True)
 
 		# Add the main textview and hook up the basename field to
 		# sync with first line of the textview
@@ -245,15 +277,14 @@ class BoundQuickNoteDialog(Dialog):
 		self.textview.get_buffer().connect('changed', self.on_text_changed)
 
 		# Initialize text from template
-		file = data_file('templates/plugins/quicknote.txt')
-		template = GenericTemplate(file.readlines(), name=file)
-		template_options.update({
-			'text': text or '',
-			'strftime': StrftimeFunction(),
-		} )
-		output = template.process(template_options)
+		template = get_template('plugins', 'quicknote.txt')
+		template_options['text'] = text or ''
+		template_options.setdefault('url', '')
+
+		lines = []
+		template.process(lines, template_options)
 		buffer = self.textview.get_buffer()
-		buffer.set_text(''.join(output))
+		buffer.set_text(''.join(lines))
 		begin, end = buffer.get_bounds()
 		buffer.place_cursor(begin)
 
@@ -288,7 +319,7 @@ class BoundQuickNoteDialog(Dialog):
 
 	def save_uistate(self):
 		self.uistate['new_page'] = self.form['new_page']
-		self.uistate['open_page'] = self.open_page.get_active()
+		self.uistate['open_page'] = self.open_page_check.get_active()
 		if self.uistate['new_page']:
 			self.uistate['namespace'] = self.form['namespace']
 		else:
@@ -310,21 +341,26 @@ class BoundQuickNoteDialog(Dialog):
 			if '\n' in title:
 				title, _ = title.split('\n', 1)
 			try:
-				title = Notebook.cleanup_pathname(title, purge=True)
+				title = Path.makeValidPageName(title.replace(':', ''))
 				self.form['basename'] = title
-			except PageNameError:
+			except ValueError:
 				pass
 			self._updating_title = False
 
-	def do_response_ok(self, get_ui=None):
-		# NOTE: Keep in mind that this method should also work using
-		# a proxy object for the ui. This is why we have the get_ui()
-		# argument to construct a proxy.
-
+	def do_response_ok(self):
 		buffer = self.textview.get_buffer()
 		bounds = buffer.get_bounds()
 		text = buffer.get_text(*bounds)
 
+		# HACK: change "[]" at start of line into "[ ]" so checkboxes get inserted correctly
+		text = re.sub(r'(?m)^(\s*)\[\](\s)', r'\1[ ]\2', text)
+		# Specify "(?m)" instead of re.M since "flags" keyword is not
+		# supported in python 2.6
+
+		notebook = self._get_notebook()
+		if notebook is None:
+			return False
+
 		if self.form['new_page']:
 			if not self.form.widgets['namespace'].get_input_valid() \
 			or not self.form['basename']:
@@ -333,44 +369,65 @@ class BoundQuickNoteDialog(Dialog):
 					entry.set_input_valid(False, show_empty_invalid=True)
 				return False
 
-			if get_ui: ui = get_ui()
-			else: ui = self.ui
-			if ui is None:
-				return False
-
-			path = self.form['namespace'].name + ':' + self.form['basename']
-			ui.new_page_from_text(text, path,
-				attachments=self.attachments,
-				open_page=self.open_page.get_active()
-			)
+			path = self.form['namespace'] + self.form['basename']
+			self.create_new_page(notebook, path, text)
 		else:
 			if not self.form.widgets['page'].get_input_valid() \
 			or not self.form['page']:
 				return False
 
-			if get_ui: ui = get_ui()
-			else: ui = self.ui
-			if ui is None:
-				return False
+			path = self.form['page']
+			self.append_to_page(notebook, path, '\n------\n'+text)
 
-			path = self.form['page'].name
-			if self.attachments:
-				ui.import_attachments(path, self.attachments)
-			ui.append_text_to_page(path, '\n----\n'+text)
-			if self.open_page.get_active():
-				ui.present(path) # also works with proxy
+		if self.attachments:
+			self.import_attachments(notebook, path)
+
+		if self.open_page_check.get_active():
+			self.hide()
+			self.open_page(notebook, path)
 
 		return True
 
+	def _get_notebook(self):
+		# Overloaded below
+		return self._ui.notebook
+
+	def create_new_page(self, notebook, path, text):
+		page = notebook.get_new_page(path)
+		page.parse('wiki', text) # FIXME format hard coded
+		notebook.store_page(page)
+
+	def append_to_page(self, notebook, path, text):
+		page = notebook.get_page(path)
+		page.parse('wiki', text, append=True) # FIXME format hard coded
+		notebook.store_page(page)
+
+	def import_attachments(self, notebook, path, folder):
+		attachments = notebook.get_attachments_dir(path)
+		for name in folder.list():
+			# FIXME could use list objects, or list_files()
+			file = dir.file(name)
+			if not file.isdir():
+				file.copyto(attachments)
+
+	def open_page(self, notebook, path):
+		assert notebook == self._ui.notebook
+		self._ui.present(path)
+
 
 class QuickNoteDialog(BoundQuickNoteDialog):
 	'''Dialog which includes a notebook chooser'''
 
-	def __init__(self, ui, notebook=None, namespace=None, basename=None, append=None, text=None, template_options=None, attachments=None):
-		self.config = get_config('quicknote.conf')
+	def __init__(self, window, notebook=None,
+		page=None, namespace=None, basename=None,
+		append=None, text=None, template_options=None, attachments=None
+	):
+		assert page is None, 'TODO'
+		manager = ConfigManager() # FIXME should be passed in
+		self.config = manager.get_config_dict('quicknote.conf')
 		self.uistate = self.config['QuickNoteDialog']
 
-		Dialog.__init__(self, ui, _('Quick Note'))
+		Dialog.__init__(self, window, _('Quick Note'))
 		self._updating_title = False
 		self._title_set_manually = not basename is None
 		self.attachments = attachments
@@ -405,7 +462,7 @@ class QuickNoteDialog(BoundQuickNoteDialog):
 		notebook = self.notebookcombobox.get_notebook()
 		self.uistate['lastnotebook'] = notebook
 		self.uistate['new_page'] = self.form['new_page']
-		self.uistate['open_page'] = self.open_page.get_active()
+		self.uistate['open_page'] = self.open_page_check.get_active()
 		if notebook is not None:
 			if self.uistate['new_page']:
 				self.config['Namespaces'][notebook] = self.form['namespace']
@@ -428,23 +485,21 @@ class QuickNoteDialog(BoundQuickNoteDialog):
 
 	def _set_autocomplete(self, notebook):
 		if notebook:
-			obj = get_notebook(notebook)
+			if isinstance(notebook, basestring):
+				notebook = NotebookInfo(notebook)
+			obj, x = build_notebook(notebook)
 			self.form.widgets['namespace'].notebook = obj
 			self.form.widgets['page'].notebook = obj
-			# Could still be None, e.g. if the notebook folder is not mounted
 			logger.debug('Notebook for autocomplete: %s (%s)', obj, notebook)
 		else:
 			self.form.widgets['namespace'].notebook = None
 			self.form.widgets['page'].notebook = None
 			logger.debug('Notebook for autocomplete unset')
 
-	def do_response_ok(self):
-		def get_ui():
-			start_server_if_not_running()
-			notebook = self.notebookcombobox.get_notebook()
-			if notebook:
-				return ServerProxy().get_notebook(notebook)
-			else:
-				return None
+	def _get_notebook(self):
+		uri = self.notebookcombobox.get_notebook()
+		notebook, p = build_notebook(Dir(uri))
+		return notebook
 
-		return BoundQuickNoteDialog.do_response_ok(self, get_ui)
+	def open_page(self, notebook, path):
+		ZIM_APPLICATION.run('--gui', notebook.uri, path.name)
